#define __CLC_BODY <clc/math/binary_decl.inc>

#define __CLC_FUNCTION nextafter
#include <clc/math/gentype.inc>
#undef __CLC_FUNCTION

#define __CLC_FUNCTION __clc_nextafter
#include <clc/math/gentype.inc>
#undef __CLC_FUNCTION

#undef __CLC_BODY
